/*
 * nbtscanner -- a tool for scanning large networks for SMB servers.
 *
 * id_list.h: Prototypes and structures for keeping track of the packet IDs.
 * Copyright (C) 2000 Steinar H. Gunderson
 *
 * Large amounts of code adapted from Samba (http://www.samba.org/)
 * Copyright (C) Andrew Tridgell 1994-1998, and others.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef _ID_LIST_H
#define _ID_LIST_H 1

#include <sys/time.h>
#include <netinet/in.h>

/* 
 * There's quite some data duplication here, in the interest of speed and code
 * simplicity.
 */

struct id_entry {
	unsigned short id;		/* ID number from NBT packet */
	int in_use;			/* Are we awaiting a reply on this ID? */

	struct in_addr sin;		/* IP address we've sent it to */
	struct timeval timestamp;	/* When we sent the packet */
	int retries;			/* Number of retries tried */

	/* we list on index entry instead of using pointers */
	int prev_index;
	int next_index;
};

struct id_list_entry {
	struct id_list_entry *next;
	struct id_entry *id;
};

/*
 * a list of all the available ID numbers (sin is unused, in_use = 0)
 *
 * We could have a used_id list as well, but it would require a doubly
 * linked list, and perhaps doesn't help all that much anyways
 */
extern struct id_list_entry *head_free_id;

/* an array with a one-to-one mapping of the array index vs. the id field */
extern struct id_entry ids[];

void id_list_init();
void id_list_destroy();
void id_free(struct id_entry *i);
void id_mark_free(struct id_entry *i);
struct id_entry *id_get_free_id();
int id_cleanup(int timeout_ms);
int get_num_free_ids();

#endif /* _ID_LIST_H */

