/*
 * nbtscanner -- a tool for scanning large networks for SMB servers.
 *
 * mysql_interface.c: Interface routines for the MySQL table. 
 * Copyright (C) 2000 Steinar H. Gunderson
 *
 * Large amounts of code adapted from Samba (http://www.samba.org/)
 * Copyright (C) Andrew Tridgell 1994-1998, and others.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <mysql.h>

#include "mysql_interface.h"
#include "configfile.h"

MYSQL *mysql = NULL;

/* veeeery detailed error reporting here ;-) */
void init_mysql(char *host, char *username, char *password)
{
	if (mysql != NULL) return;	/* hack */

	mysql = mysql_init(NULL);
	if (mysql == NULL) {
		if (verbosity >= 1) {
			fprintf(stderr, "mysql_init() failed\n");
		}
		exit(1);
	}

	if (mysql_real_connect(mysql, mysql_host, username, password, "nbtscanner",
		0, NULL, 0) == 0) {
		if (verbosity >= 1) {
			fprintf(stderr, "mysql_real_connect() failed\n");
		}
		exit(1);
	}
}

void add_record_mysql(char *ip, char *hostname, char *fileservername, char *groupname)
{
	char query[256];
	char hname[256], fsname[256], gname[256];

	mysql_real_escape_string(mysql, hname, hostname, strlen(hostname));
	mysql_real_escape_string(mysql, fsname, fileservername, strlen(fileservername));
	mysql_real_escape_string(mysql, gname, groupname, strlen(groupname));

	snprintf(query, 256,
		"REPLACE INTO nbtscanner.nbtscanner SET ip='%s', hostname='%s', fileservername='%s', groupname='%s';",
		ip, hname, fsname, gname);

	if (mysql_query(mysql, query) != 0) {
		if (verbosity >= 1) {
			fprintf(stderr, "mysql_query(`%s') failed\n", query);
		}
		exit(1);
	}
}

void delete_record_mysql(char *ip)
{
	char query[256];
	snprintf(query, 256,
		"DELETE FROM nbtscanner.nbtscanner WHERE ip='%s';",
		ip);

	if (mysql_query(mysql, query) != 0) {
		if (verbosity >= 1) {
			fprintf(stderr, "mysql_query(`%s') failed\n", query);
		}
		exit(1);
	}
}

void print_all_records_mysql()
{
	MYSQL_RES *result;	
	unsigned long *lengths;
	unsigned int num_fields;
	int i;

	if (mysql_query(mysql, "SELECT ip,(CASE WHEN fileservername='-unknown-nbtscanner-' THEN hostname ELSE fileservername END),groupname,(fileservername <> '-unknown-nbtscanner-') FROM nbtscanner.nbtscanner;") != 0) {
		if (verbosity >= 1) {
			fprintf(stderr, "mysql_query('SELECT ip'...) failed\n");
		}
		exit(1);
	}

	result = mysql_store_result(mysql);
	if (result == NULL) {
		if (verbosity >= 1) {
			fprintf(stderr, "mysql_store_result() failed\n");
		}
		exit(1);
	}

	num_fields = mysql_num_fields(result);
	while (1) {
		MYSQL_ROW row = mysql_fetch_row(result);
		if (row == NULL) return;

		lengths = mysql_fetch_lengths(result);

        	for (i = 0; i < num_fields; i++) {
			int j;

			/* strip away CR/LFs */
			for (j = 0; j < lengths[i]; j++) {
				if (row[i][j] == 0 || row[i][j] == 10 || row[i][j] == 13 || row[i][j] == ',') {
					row[i][j] = '?';
				}
			}
			printf("%.*s", (int) lengths[i], row[i] ? row[i] : "NULL");
			if (i != num_fields - 1) printf(",");
		}
		printf("\r\n");
        }
}

void finish_mysql()
{
	/* don't care if this fails */
	mysql_query(mysql, "OPTIMIZE TABLE nbtscanner.nbtscanner;");

	mysql_close(mysql);
}

