/*
 * nbtscanner -- a tool for scanning large networks for SMB servers.
 *
 * stats.c: Running statistics.
 * Copyright (C) 2000 Steinar H. Gunderson
 *
 * Large amounts of code adapted from Samba (http://www.samba.org/)
 * Copyright (C) Andrew Tridgell 1994-1998, and others.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "stats.h"
#include "configfile.h"
#include "util.h"

#include <stdio.h>
#include <sys/time.h>

int num_recv;
int num_recv_retries;
int num_failed;
int num_sent;
int num_sent_total;
enum statuses status;

struct timeval last_stats;

void init_stats()
{
	num_recv = 0;
	num_recv_retries = 0;
	num_failed = 0;
	num_sent = 0;
	num_sent_total = 0;
	status = SCANNING;

	gettimeofday(&last_stats, NULL);
}

void print_stats(int force)
{
	struct timeval now;

	gettimeofday(&now, NULL);
	if (verbosity >= 3 && (mydifftime(last_stats, now) > 1000 || force == 1)) {
		fprintf(stderr, "%s: Received %5u/%5u/%5u (%5u failed) (avg. "
				"retries/succ.: %3.2f)\r",
			(status == SCANNING) ? "Scanning" : "Retrying",
			num_recv, num_sent, num_sent_total, num_failed, 
			(float)num_recv_retries / (float)num_recv);

		gettimeofday(&last_stats, NULL);
	}
}

